@@ -4,11 +4,11 @@ use cargo::{
     core::registry::PackageRegistry,
     core::source::MaybePackage,
     core::{
-        Dependency, EitherManifest, FeatureValue, Manifest, Package, PackageId, Registry, Source,
-        SourceId, Summary, Target, TargetKind, Workspace,
+        resolver::features::CliFeatures, Dependency, EitherManifest, FeatureValue, Manifest,
+        Package, PackageId, Registry, Source, SourceId, Summary, Target, TargetKind, Workspace,
     },
     ops,
-    ops::PackageOpts,
+    ops::{PackageOpts, Packages},
     sources::registry::RegistrySource,
     util::{interning::InternedString, toml::read_manifest, FileLock},
     Config,
@@ -26,13 +26,15 @@ use std::collections::{BTreeMap, HashSet};
 use std::fs;
 use std::hash::{Hash, Hasher};
 use std::io::{self, Read, Write};
-use std::path::{Path, PathBuf};
+use std::path::Path;
 
+use crate::config::testing_ignore_debpolv;
 use crate::errors::*;
-use crate::util::vec_opt_iter;
 
 pub struct CrateInfo {
+    // only used for to_registry_toml in extract_crate. DO NOT USE ELSEWHERE
     package: Package,
+    // allows overriding package.manifest() e.g. via patches
     manifest: Manifest,
     crate_file: FileLock,
     config: Config,
@@ -41,6 +43,15 @@ pub struct CrateInfo {
     includes: Vec<Pattern>,
 }
 
+pub type CrateDepInfo = BTreeMap<
+    &'static str, // name of feature / optional dependency,
+    // or "" for the base package w/ no default features, guaranteed to be in the map
+    (
+        Vec<&'static str>, // dependencies: other features (of the current package)
+        Vec<Dependency>,
+    ),
+>;
+
 fn hash<H: Hash>(hashable: &H) -> u64 {
     #![allow(deprecated)]
     let mut hasher = std::hash::SipHasher::new();
@@ -63,6 +74,26 @@ pub fn update_crates_io() -> Result<()> {
     r.update()
 }
 
+pub fn crate_name_ver_to_dep(crate_name: &str, version: Option<&str>) -> Result<Dependency> {
+    // note: this forces a network call
+    let config = Config::default()?;
+    let source_id = SourceId::crates_io(&config)?;
+    let version = version.and_then(|v| {
+        if v.is_empty() {
+            None
+        } else if v.starts_with(|c: char| c.is_digit(10)) {
+            Some(["=", v].concat())
+        } else {
+            Some(v.to_string())
+        }
+    });
+    Dependency::parse(crate_name, version.as_deref(), source_id)
+}
+
+pub fn show_dep(dep: &Dependency) -> String {
+    format!("{} {}", dep.package_name(), dep.version_req())
+}
+
 impl CrateInfo {
     pub fn new(crate_name: &str, version: Option<&str>) -> Result<CrateInfo> {
         CrateInfo::new_with_update(crate_name, version, true)
@@ -85,7 +116,7 @@ impl CrateInfo {
 
             let package_id = match version {
                 None | Some("") => {
-                    let dep = Dependency::parse_no_deprecated(crate_name, None, source_id)?;
+                    let dep = Dependency::parse(crate_name, None, source_id)?;
                     let mut package_id: Option<PackageId> = None;
                     source.query(&dep, &mut |p| package_id = Some(p.package_id()))?;
                     package_id.unwrap()
@@ -112,11 +143,10 @@ impl CrateInfo {
                     list: false,
                     check_metadata: true,
                     allow_dirty: true,
-                    all_features: true,
-                    no_default_features: false,
+                    cli_features: CliFeatures::from_command_line(&[], true, false)?,
                     jobs: None,
                     targets: Vec::new(),
-                    features: Vec::new(),
+                    to_package: Packages::Default,
                 };
 
                 // as of cargo 0.41 this returns a FileLock with a temp path, instead of the one
@@ -135,7 +165,7 @@ impl CrateInfo {
                     .open_rw(&filename, &config, "crate file")?
             };
 
-            (package.clone(), crate_file)
+            (package, crate_file)
         };
 
         let manifest = package.manifest().clone();
@@ -156,44 +186,39 @@ impl CrateInfo {
         version: Option<&str>,
         update: bool,
     ) -> Result<CrateInfo> {
-        let config = Config::default()?;
-        let source_id = {
-            let source_id = SourceId::crates_io(&config)?;
-            if update {
-                source_id
-            } else {
-                // The below is a bit of a hack and depends on some cargo internals
-                // but unless we do this, fetch_candidates() will update the index
-                // The behaviour is brittle; we really should write a test for it.
-                source_id.with_precise(Some("locked".to_string()))
-            }
-        };
+        let dep = crate_name_ver_to_dep(crate_name, version)?;
+        Self::new_from_dependency(&dep, update)
+    }
 
-        let version = version.map(|v| {
-            if v.starts_with(|c: char| c.is_digit(10)) {
-                ["=", v].concat()
-            } else {
-                v.to_string()
-            }
-        });
-
-        let dependency = Dependency::parse_no_deprecated(
-            crate_name,
-            version.as_ref().map(String::as_str),
-            source_id,
-        )?;
+    pub fn new_from_dependency(dependency: &Dependency, update: bool) -> Result<CrateInfo> {
+        let mut config = Config::default()?;
+        if !update {
+            // unfriendly API from cargo; we'll have to make do with it for
+            // now as there is no other alternative
+            config.configure(
+                0,
+                false,
+                None,
+                config.frozen(),
+                config.locked(),
+                true, // offline
+                &config.target_dir()?.map(|x| x.into_path_unlocked()),
+                &[],
+                &[],
+            )?;
+        }
 
+        let source_id = dependency.source_id();
         let registry_name = format!(
             "{}-{:016x}",
             source_id.url().host_str().unwrap_or(""),
             hash(&source_id).swap_bytes()
         );
-
-        let (package, manifest, crate_file) = {
+        let get_package_info = |config: &Config| -> Result<_> {
             let lock = config.acquire_package_cache_lock()?;
-            let mut registry = PackageRegistry::new(&config)?;
+            let mut registry = PackageRegistry::new(config)?;
             registry.lock_patches();
-            let summaries = fetch_candidates(&mut registry, &dependency)?;
+            let summaries = fetch_candidates(&mut registry, dependency)?;
             drop(lock);
             let pkgids = summaries
                 .into_iter()
@@ -202,23 +227,43 @@ impl CrateInfo {
             let pkgid = pkgids.iter().max().ok_or_else(|| {
                 format_err!(
                     concat!(
-                        "Couldn't find any crate matching {} {}\n ",
+                        "Couldn't find any crate matching {}\n",
                         "Try `debcargo update` to update the crates.io index."
                     ),
-                    dependency.package_name(),
-                    dependency.version_req()
+                    show_dep(dependency)
                 )
             })?;
             let pkgset = registry.get(pkgids.as_slice())?;
             let package = pkgset.get_one(*pkgid)?;
             let manifest = package.manifest();
+            for f in dependency.features() {
+                // apparently, if offline is set then cargo sometimes selects
+                // an offline-available version that doesn't satisfy the
+                // requested features. this is dumb. if it happens, then we
+                // retry with online allowed.
+                if !manifest.summary().features().contains_key(f) {
+                    debcargo_bail!(
+                        "resolve ({} {}) -> ({}) failed to pick up required feature ({})\n\
+                        This can happen with very old or yanked crates. Try patching one of \
+                        its dependants, to drop or update the offending dependency.",
+                        dependency.package_name(),
+                        dependency.version_req(),
+                        pkgid,
+                        f,
+                    )
+                }
+            }
             let filename = format!("{}-{}.crate", pkgid.name(), pkgid.version());
             let crate_file = config
                 .registry_cache_path()
                 .join(&registry_name)
-                .open_ro(&filename, &config, &filename)?;
-            (package.clone(), manifest.clone(), crate_file)
+                .open_ro(&filename, config, &filename)?;
+            Ok((package.clone(), manifest.clone(), crate_file))
         };
+        // if update is false but the user never downloaded the crate then the
+        // first call will error; re-try with online in that case
+        let (package, manifest, crate_file) =
+            get_package_info(&config).or_else(|_| get_package_info(&Config::default()?))?;
 
         Ok(CrateInfo {
             package,
@@ -231,8 +276,8 @@ impl CrateInfo {
         })
     }
 
-    pub fn package(&self) -> &Package {
-        &self.package
+    pub fn crate_name(&self) -> &'static str {
+        self.package_id().name().as_str()
     }
 
     pub fn version(&self) -> &Version {
@@ -252,7 +297,7 @@ impl CrateInfo {
         &self.manifest
     }
 
-    pub fn replace_manifest(&mut self, path: &PathBuf) -> Result<&Self> {
+    pub fn replace_manifest(&mut self, path: &Path) -> Result<&Self> {
         if let (EitherManifest::Real(v), _) = read_manifest(path, self.source_id, &self.config)? {
             self.manifest = v;
         }
@@ -263,6 +308,10 @@ impl CrateInfo {
         self.manifest.metadata()
     }
 
+    pub fn manifest_path(&self) -> &Path {
+        self.package.manifest_path()
+    }
+
     pub fn targets(&self) -> &[Target] {
         self.manifest.targets()
     }
@@ -291,7 +340,7 @@ impl CrateInfo {
                 _ => continue,
             }
         }
-        bins.sort();
+        bins.sort_unstable();
         bins
     }
 
@@ -328,17 +377,7 @@ impl CrateInfo {
 
     /// Collect information about the dependency structure of features and
     /// their external crate dependencies, in a simple output format.
-    pub fn all_dependencies_and_features(
-        &self,
-    ) -> BTreeMap<
-        &str, // name of feature / optional dependency,
-        // or "" for the base package w/ no default features, guaranteed to be in the map
-        (
-            Vec<&str>, // dependencies: other features (of the current package)
-            Vec<Dependency>,
-        ),
-    > // dependencies: other packages
-    {
+    pub fn all_dependencies_and_features(&self) -> CrateDepInfo {
         use cargo::core::dependency::DepKind;
 
         let mut deps_by_name: BTreeMap<&str, Vec<&Dependency>> = BTreeMap::new();
@@ -355,30 +394,63 @@ impl CrateInfo {
 
         // calculate dependencies of this crate's features
         for (feature, deps) in self.manifest.summary().features() {
-            let mut feature_deps: Vec<&str> = vec![];
+            let mut feature_deps: Vec<&'static str> = vec![];
             let mut other_deps: Vec<Dependency> = Vec::new();
             for dep in deps {
                 use self::FeatureValue::*;
                 match dep {
                     // another feature is a dependency
-                    Feature(dep_feature) => feature_deps.push(dep_feature),
+                    Feature(dep_feature) => {
+                        feature_deps.push(InternedString::new(dep_feature).as_str())
+                    }
                     // another package is a dependency
-                    Crate(dep_name) => {
+                    Dep { dep_name } => {
                         // unwrap is ok, valid Cargo.toml files must have this
                         for &dep in deps_by_name.get(dep_name.as_str()).unwrap() {
                             other_deps.push(dep.clone());
                         }
                     }
                     // another package is a dependency
-                    CrateFeature(dep_name, dep_feature) => {
+                    DepFeature {
+                        dep_name,
+                        dep_feature,
+                        ..
+                    } => match deps_by_name.get(dep_name.as_str()) {
                         // unwrap is ok, valid Cargo.toml files must have this
-                        for &dep in deps_by_name.get(dep_name.as_str()).unwrap() {
-                            let mut dep = dep.clone();
-                            dep.set_features(vec![InternedString::new(dep_feature)]);
-                            dep.set_default_features(false);
-                            other_deps.push(dep);
+                        Some(dd) => {
+                            for &dep in dd {
+                                let mut dep = dep.clone();
+                                dep.set_features(vec![InternedString::new(dep_feature)]);
+                                dep.set_default_features(false);
+                                other_deps.push(dep);
+                            }
                         }
-                    }
+                        None => {
+                            let mut expected = false;
+                            for dep in self.dependencies() {
+                                if dep.kind() == DepKind::Development {
+                                    let s = dep.name_in_toml().as_str();
+                                    if s == dep_name.as_str() {
+                                        expected = true;
+                                    }
+                                }
+                            }
+                            if expected {
+                                debcargo_warn!(
+                                    "Ignoring \"{}\" feature \"{}\" as it depends on a \
+                                     dev-dependency \"{}\"",
+                                    self.package_id(),
+                                    feature,
+                                    dep_name
+                                );
+                            } else {
+                                panic!(
+                                    "failed to account for dependency \"{}\" of \"{}\" feature \"{}\"",
+                                    dep_name, self.package_id(), feature
+                                );
+                            }
+                        }
+                    },
                 }
             }
             if feature_deps.is_empty() {
@@ -394,7 +466,7 @@ impl CrateInfo {
             for &dep in deps {
                 if dep.is_optional() {
                     features_with_deps
-                        .insert(&dep.name_in_toml().as_str(), (vec![""], vec![dep.clone()]));
+                        .insert(dep.name_in_toml().as_str(), (vec![""], vec![dep.clone()]));
                 } else {
                     deps_required.push(dep.clone())
                 }
@@ -467,15 +539,20 @@ impl CrateInfo {
         (summary, description)
     }
 
+    /// To be called before extract_crate.
     pub fn set_includes_excludes(
         &mut self,
         excludes: Option<&Vec<String>>,
         includes: Option<&Vec<String>>,
     ) {
-        self.excludes = vec_opt_iter(excludes)
+        self.excludes = excludes
+            .into_iter()
+            .flatten()
             .map(|x| Pattern::new(&("*/".to_owned() + x)).unwrap())
             .collect::<Vec<_>>();
-        self.includes = vec_opt_iter(includes)
+        self.includes = includes
+            .into_iter()
+            .flatten()
             .map(|x| Pattern::new(&("*/".to_owned() + x)).unwrap())
             .collect::<Vec<_>>();
     }
@@ -492,6 +569,9 @@ impl CrateInfo {
             if self.includes.iter().any(|p| p.matches_path(path)) {
                 debcargo_info!("Suspicious file, on whitelist so ignored: {:?}", path);
                 Ok(false)
+            } else if testing_ignore_debpolv() {
+                debcargo_warn!("Suspicious file, ignoring as per override: {:?}", path);
+                Ok(false)
             } else {
                 Err(format!(
                     "Suspicious file, should probably be excluded: {:?}",
@@ -554,20 +634,20 @@ impl CrateInfo {
         }
 
         if let Err(e) = fs::rename(entries[0].path(), &path) {
-            return Err(Error::from(Error::from(e).context(format!(
+            return Err(Error::from(e).context(format!(
                 concat!(
                     "Could not create source directory {0}\n",
                     "To regenerate, move or remove {0}"
                 ),
                 path.display()
-            ))));
+            )));
         }
 
         // Ensure that Cargo.toml is in standard form, e.g. does not contain
         // path dependencies, so can be built standalone (see #4030).
         let toml_path = path.join("Cargo.toml");
         let ws = Workspace::new(&toml_path.canonicalize()?, &self.config)?;
-        let registry_toml = self.package().to_registry_toml(&ws)?;
+        let registry_toml = self.package.to_registry_toml(&ws)?;
         let mut actual_toml = String::new();
         fs::File::open(&toml_path)?.read_to_string(&mut actual_toml)?;
 
@@ -598,3 +678,23 @@ impl CrateInfo {
         Ok(source_modified)
     }
 }
+
+/// Calculate all feature-dependencies and external-dependencies of a given
+/// feature, using the information previously generated by
+/// `all_dependencies_and_features`.
+pub fn transitive_deps<'a>(
+    features_with_deps: &'a CrateDepInfo,
+    feature: &str,
+) -> (Vec<&'a str>, Vec<Dependency>) {
+    let mut all_features = Vec::new();
+    let mut all_deps = Vec::new();
+    let &(ref ff, ref dd) = features_with_deps.get(feature).unwrap();
+    all_features.extend(ff.clone());
+    all_deps.extend(dd.clone());
+    for f in ff {
+        let (ff1, dd1) = transitive_deps(features_with_deps, f);
+        all_features.extend(ff1);
+        all_deps.extend(dd1);
+    }
+    (all_features, all_deps)
+}
