@@ -5,9 +5,8 @@ use anyhow::{format_err, Error};
 use semver::Version;
 use textwrap::fill;
 
-use crate::config::{package_field_for_feature, Config, PackageKey};
+use crate::config::{self, Config, PackageKey};
 use crate::errors::*;
-use crate::util::vec_opt_iter;
 
 pub struct Source {
     name: String,
@@ -21,6 +20,7 @@ pub struct Source {
     vcs_browser: String,
     homepage: String,
     x_cargo: String,
+    requires_root: String,
 }
 
 pub struct Package {
@@ -32,11 +32,16 @@ pub struct Package {
     recommends: Vec<String>,
     suggests: Vec<String>,
     provides: Vec<String>,
-    summary: String,
-    description: String,
+    summary: Description,
+    description: Description,
     extra_lines: Vec<String>,
 }
 
+pub struct Description {
+    pub prefix: String,
+    pub suffix: String,
+}
+
 pub struct PkgTest {
     name: String,
     crate_name: String,
@@ -68,7 +73,7 @@ impl fmt::Display for Source {
         if !self.x_cargo.is_empty() {
             writeln!(f, "X-Cargo-Crate: {}", self.x_cargo)?;
         }
-        writeln!(f, "Rules-Requires-Root: no")?;
+        writeln!(f, "Rules-Requires-Root: {}", self.requires_root)?;
 
         Ok(())
     }
@@ -143,6 +148,16 @@ impl fmt::Display for PkgTest {
 }
 
 impl Source {
+    pub fn pkg_prefix() -> &'static str {
+        if config::testing_ruzt() {
+            // avoid accidentally installing official packages during tests
+            "ruzt"
+        } else {
+            "rust"
+        }
+    }
+
+    #[allow(clippy::too_many_arguments)]
     pub fn new(
         basename: &str,
         name_suffix: Option<&str>,
@@ -152,6 +167,7 @@ impl Source {
         maintainer: String,
         uploaders: Vec<String>,
         build_deps: Vec<String>,
+        _requires_root: String,
     ) -> Result<Source> {
         let pkgbase = match name_suffix {
             None => basename.to_string(),
@@ -172,7 +188,7 @@ impl Source {
             pkgbase
         );
 
-        let cargo_crate = if upstream_name != upstream_name.replace('_', "-") {
+        let cargo_crate = if upstream_name != base_deb_name(upstream_name) {
             upstream_name.to_string()
         } else {
             "".to_string()
@@ -189,9 +205,14 @@ impl Source {
             vcs_browser,
             homepage: home.to_string(),
             x_cargo: cargo_crate,
+            requires_root: "no".to_string(),
         })
     }
 
+    pub fn name(&self) -> &str {
+        &self.name
+    }
+
     pub fn apply_overrides(&mut self, config: &Config) {
         if let Some(section) = config.section() {
             self.section = section.to_string();
@@ -201,8 +222,13 @@ impl Source {
             self.standards = policy.to_string();
         }
 
-        self.build_deps
-            .extend(vec_opt_iter(config.build_depends()).map(String::to_string));
+        self.build_deps.extend(
+            config
+                .build_depends()
+                .into_iter()
+                .flatten()
+                .map(String::to_string),
+        );
         let bdeps_ex = config
             .build_depends_excludes()
             .map(Vec::as_slice)
@@ -220,16 +246,30 @@ impl Source {
         if let Some(vcs_browser) = config.vcs_browser() {
             self.vcs_browser = vcs_browser.to_string();
         }
+
+        if let Some(requires_root) = config.requires_root() {
+            self.requires_root = requires_root.to_string();
+        }
     }
 }
 
 impl Package {
+    pub fn pkg_prefix() -> &'static str {
+        if config::testing_ruzt() {
+            // avoid accidentally installing official packages during tests
+            "libruzt"
+        } else {
+            "librust"
+        }
+    }
+
+    #[allow(clippy::too_many_arguments)]
     pub fn new(
         basename: &str,
         name_suffix: Option<&str>,
         version: &Version,
-        summary: &str,
-        description: &str,
+        summary: Description,
+        description: Description,
         feature: Option<&str>,
         f_deps: Vec<&str>,
         o_deps: Vec<String>,
@@ -250,7 +290,7 @@ impl Package {
                 }
             )
         };
-        let deb_feature = &|f: &str| deb_feature2(&pkgbase, &f);
+        let deb_feature = &|f: &str| deb_feature2(&pkgbase, f);
 
         let filter_provides = &|x: Vec<&str>| {
             x.into_iter()
@@ -279,7 +319,8 @@ impl Package {
             provides.extend(f_provides.iter().map(|f| deb_feature2(&p, f)));
         }
         let provides_self = deb_feature(feature.unwrap_or(""));
-        // TODO: can use remove_item() when that is stabilised
+        // rust dropped Vec::remove_item for annoying reasons, the below is
+        // an unofficial recommended replacement from the RFC #40062
         let i = provides.iter().position(|x| *x == *provides_self);
         i.map(|i| provides.remove(i));
 
@@ -325,8 +366,8 @@ impl Package {
             recommends,
             suggests,
             provides,
-            summary: summary.to_string(),
-            description: description.to_string(),
+            summary,
+            description,
             extra_lines: match (name_suffix, feature) {
                 (Some(_), None) => {
                     let fullpkg = format!("{}-{}", basename, version);
@@ -344,8 +385,8 @@ impl Package {
         basename: &str,
         name_suffix: Option<&str>,
         section: Option<&str>,
-        summary: &str,
-        description: &str,
+        summary: Description,
+        description: Description,
     ) -> Self {
         let (name, mut provides) = match name_suffix {
             None => (basename.to_string(), vec![]),
@@ -368,8 +409,8 @@ impl Package {
             recommends: vec!["${cargo:Recommends}".to_string()],
             suggests: vec!["${cargo:Suggests}".to_string()],
             provides,
-            summary: summary.to_string(),
-            description: description.to_string(),
+            summary,
+            description,
             extra_lines: vec![
                 "Built-Using: ${cargo:Built-Using}".to_string(),
                 "XB-X-Cargo-Built-Using: ${cargo:X-Cargo-Built-Using}".to_string(),
@@ -382,8 +423,9 @@ impl Package {
     }
 
     fn write_description(&self, out: &mut fmt::Formatter) -> fmt::Result {
-        writeln!(out, "Description: {}", self.summary)?;
-        for line in fill(&self.description.trim(), 79).lines() {
+        writeln!(out, "Description: {}", &self.summary)?;
+        let description = format!("{}", &self.description);
+        for line in fill(description.trim(), 79).lines() {
             let line = line.trim_end();
             if line.is_empty() {
                 writeln!(out, " .")?;
@@ -396,40 +438,68 @@ impl Package {
         Ok(())
     }
 
+    #[allow(clippy::result_unit_err)]
+    pub fn summary_check_len(&self) -> std::result::Result<(), ()> {
+        if self.summary.prefix.len() <= 80 {
+            Ok(())
+        } else {
+            Err(())
+        }
+    }
+
     pub fn apply_overrides(&mut self, config: &Config, key: PackageKey, f_provides: Vec<&str>) {
         if let Some(section) = config.package_section(key) {
             self.section = Some(section.to_string());
         }
-        if let Some(summary) = config.package_summary(key) {
-            self.summary = summary.to_string();
-        }
-        if let Some(description) = config.package_description(key) {
-            self.description = description.to_string();
-        }
+        self.summary
+            .apply_overrides(&config.summary, config.package_summary(key));
+        self.description
+            .apply_overrides(&config.description, config.package_description(key));
 
-        self.depends.extend(package_field_for_feature(
+        self.depends.extend(config::package_field_for_feature(
             &|x| config.package_depends(x),
             key,
             &f_provides,
         ));
-        self.recommends.extend(package_field_for_feature(
+        self.recommends.extend(config::package_field_for_feature(
             &|x| config.package_recommends(x),
             key,
             &f_provides,
         ));
-        self.suggests.extend(package_field_for_feature(
+        self.suggests.extend(config::package_field_for_feature(
             &|x| config.package_suggests(x),
             key,
             &f_provides,
         ));
-        self.provides.extend(package_field_for_feature(
+        self.provides.extend(config::package_field_for_feature(
             &|x| config.package_provides(x),
             key,
             &f_provides,
         ));
 
-        self.extra_lines
-            .extend(vec_opt_iter(config.package_extra_lines(key)).map(|s| s.to_string()));
+        self.extra_lines.extend(
+            config
+                .package_extra_lines(key)
+                .into_iter()
+                .flatten()
+                .map(|s| s.to_string()),
+        );
+    }
+}
+
+impl Description {
+    fn apply_overrides(&mut self, global: &Option<String>, per_package: Option<&str>) {
+        if let Some(per_package) = per_package {
+            self.prefix = per_package.to_string();
+            self.suffix = "".to_string();
+        } else if let Some(global) = &global {
+            self.prefix = global.into();
+        }
+    }
+}
+impl fmt::Display for Description {
+    fn fmt(&self, f: &mut fmt::Formatter) -> fmt::Result {
+        write!(f, "{}{}", &self.prefix, self.suffix)
     }
 }
 
@@ -455,29 +525,35 @@ impl PkgTest {
     }
 }
 
-/// Translates a semver into a Debian version. Omits the build metadata, and uses a ~ before the
-/// prerelease version so it compares earlier than the subsequent release.
-pub fn deb_version(v: &Version) -> String {
+/// Translates a semver into a Debian-format upstream version.
+/// Omits the build metadata, and uses a ~ before the prerelease version so it
+/// compares earlier than the subsequent release.
+pub fn deb_upstream_version(v: &Version) -> String {
     let mut s = format!("{}.{}.{}", v.major, v.minor, v.patch);
-    for (n, id) in v.pre.iter().enumerate() {
-        write!(s, "{}{}", if n == 0 { '~' } else { '.' }, id).unwrap();
+    if !v.pre.is_empty() {
+        write!(s, "~{}", v.pre.as_str()).unwrap();
     }
     s
 }
 
+pub fn base_deb_name(crate_name: &str) -> String {
+    crate_name.replace('_', "-").to_lowercase()
+}
+
 pub fn dsc_name(name: &str) -> String {
-    format!("rust-{}", name.replace('_', "-"))
+    format!("{}-{}", Source::pkg_prefix(), base_deb_name(name))
 }
 
 pub fn deb_name(name: &str) -> String {
-    format!("librust-{}-dev", name.replace('_', "-"))
+    format!("{}-{}-dev", Package::pkg_prefix(), base_deb_name(name))
 }
 
 pub fn deb_feature_name(name: &str, feature: &str) -> String {
     format!(
-        "librust-{}+{}-dev",
-        name.replace('_', "-"),
-        feature.replace('_', "-").to_lowercase()
+        "{}-{}+{}-dev",
+        Package::pkg_prefix(),
+        base_deb_name(name),
+        base_deb_name(feature)
     )
 }
 
@@ -490,10 +566,8 @@ fn get_envs(keys: &[&str]) -> Result<Option<String>> {
                 return Ok(Some(val));
             }
             Err(e @ VarError::NotUnicode(_)) => {
-                return Err(Error::from(
-                    Error::from(e)
-                        .context(format!("Environment variable ${} not valid UTF-8", key)),
-                ));
+                return Err(Error::from(e)
+                    .context(format!("Environment variable ${} not valid UTF-8", key)));
             }
             Err(VarError::NotPresent) => {}
         }
