@@ -1,4 +1,4 @@
-use cargo::core::{manifest, package};
+use cargo::core::manifest;
 use chrono::{DateTime, Datelike, NaiveDateTime, Utc};
 use git2::Repository;
 use regex;
@@ -16,8 +16,7 @@ use std::path::Path;
 
 use crate::errors::*;
 
-const DEB_COPYRIGHT_FORMAT: &str =
-    "\
+const DEB_COPYRIGHT_FORMAT: &str = "\
      https://www.debian.org/doc/packaging-manuals/copyright-format/1.0/";
 
 macro_rules! format_para {
@@ -335,9 +334,9 @@ fn copyright_fromgit(repo_url: &str) -> Result<String> {
 }
 
 pub fn debian_copyright(
-    package: &package::Package,
     srcdir: &Path,
     manifest: &manifest::Manifest,
+    manifest_path: &Path,
     maintainer: &str,
     uploaders: &[&str],
     year_range: (i32, i32),
@@ -349,13 +348,23 @@ pub fn debian_copyright(
         Some(ref r) => r,
     };
 
-    let upstream = UpstreamInfo::new(manifest.name().to_string(), &meta.authors, repository);
+    // The Authors field is optional according to
+    // https://rust-lang.github.io/rfcs/3052-optional-authors-field.html
+    // and crates.io publishes crates without the field already.
+    let unknown_authors = vec!["FIXME (overlay) UNKNOWN-AUTHORS".to_string()];
+    let authors = if meta.authors.is_empty() {
+        &unknown_authors
+    } else {
+        &meta.authors
+    };
+
+    let upstream = UpstreamInfo::new(manifest.name().to_string(), authors, repository);
 
     let mut licenses: Vec<License> = Vec::new();
     let mut crate_license: String = "".to_string();
 
     if let Some(ref license_file_name) = meta.license_file {
-        let license_file = package.manifest_path().with_file_name(license_file_name);
+        let license_file = manifest_path.with_file_name(license_file_name);
         let mut text = Vec::new();
         fs::File::open(license_file)?.read_to_end(&mut text)?;
         licenses.reserve(1);
@@ -405,6 +414,7 @@ pub fn debian_copyright(
         "FIXME (overlay) UNKNOWN-YEARS".to_string()
     };
     let notice = match meta.authors.len() {
+        0 => vec![format!("FIXME (overlay) UNKNOWN-AUTHORS {}", years)],
         1 => vec![format!("{} {}", years, &meta.authors[0])],
         _ => meta
             .authors
