@@ -8,7 +8,7 @@ macro_rules! debcargo_info {
     ($e:expr) => {
         {
             use ansi_term::Colour::Green;
-            println!("{}", Green.paint($e));
+            eprintln!("{}", Green.paint($e));
         }
     };
 
@@ -16,7 +16,7 @@ macro_rules! debcargo_info {
         {
             use ansi_term::Colour::Green;
             let print_string = format!($fmt, $($arg)+);
-            println!("{}", Green.paint(print_string));
+            eprintln!("{}", Green.paint(print_string));
         }
     };
 }
@@ -26,7 +26,7 @@ macro_rules! debcargo_warn {
     ($e:expr) => {
         {
             use ansi_term::Colour::RGB;
-            println!("{}", RGB(255,165,0).bold().paint($e));
+            eprintln!("{}", RGB(255,165,0).bold().paint($e));
         }
     };
 
@@ -34,7 +34,7 @@ macro_rules! debcargo_warn {
         {
             use ansi_term::Colour::RGB;
             let print_string = RGB(255,165,0).bold().paint(format!($fmt, $($arg)+));
-            println!("{}", print_string);
+            eprintln!("{}", print_string);
         }
     };
 
@@ -43,13 +43,13 @@ macro_rules! debcargo_warn {
 #[macro_export]
 macro_rules! debcargo_bail {
     ($e:expr) => {{
-        return Err(format_err!("{}", $e));
+        return Err(::anyhow::format_err!("{}", $e));
     }};
 
     ($fmt:expr, $( $arg:tt)+) => {
         {
             let error_string = format!($fmt, $($arg)+);
-            return Err(format_err!("{}", error_string));
+            return Err(::anyhow::format_err!("{}", error_string));
         }
     };
 }
